#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import os, string, sys
import math
from array import array

#get pixel helper function
def get_pixel(point,image_width,p_size,src_pixels) :
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	pix = src_pixels[src_pos: src_pos + p_size]
	if p_size == 3:    #if it doesn't have alpha information add full opacity
		pix.append(255)
	return (pix)
def set_pixel(point,image_width,p_size,src_pixels,color):
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	#src_pixels[src_pos: src_pos + p_size] = array("B",color).tostring()
	src_pixels[src_pos] = color[0]
	src_pixels[src_pos+1] = color[1]
	src_pixels[src_pos+2] = color[2]
	src_pixels[src_pos+3] = color[3]
	return None
	
def distance(p1,p2):
	return (((p2[0] - p1[0])**2 + (p2[1] - p1[1])**2)**0.5)
	
def height_stretch_selection(image,layer,new_bright_value,new_dark_value):
	pdb.gimp_image_undo_group_start(image)
	
	pdb.gimp_layer_resize_to_image_size(layer) # make layer same as image size to not deal with possible layer offsets
	#pdb.gimp_layer_resize_to_image_size(layer2)
	
	#creates a new layer and name it
	#new_layer = gimp.Layer(image, "Intersect Raster", image.width, image.height, RGBA_IMAGE,
    #                       100, NORMAL_MODE)
	#image.add_layer(new_layer, 0)
	#couldn't call these below properly not sure why. but found above code online on some examples
	#new_layer = pdb.gimp_layer_new(image,image.width,image.height,RGB_IMAGE,"Intersect raster",100,NORMAL_MODE)
	#pdb.gimp_image_insert_layer(image,new_layer,0,0)
	
	#grab layer's pixel information
	
	#gets selection value at 10,10
	#pdb.gimp_message(str(image.selection.get_pixel(10,10)[0]))
	
	srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height, True, False)
	p_size = len(srcRgn[0,0])
	
	src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height])
	
	if pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_message("Layer has alpha. OK!")
	else:
		raise Exception("You need to add alpha channel to layer!")
	#selRgn = image.selection.get_pixel_rgn(0, 0, layer.width, layer.height, True, False)
	#pdb.gimp_message(str(image.selection))
	
	selection = pdb.gimp_selection_bounds(image)
	selected = selection[0]
	if selected < 1:
		pdb.gimp_message("You need to make a selection!")
		raise Exception("You need to make a selection!")
		
	
	x1 = selection[1]
	y1 = selection[2]
	x2 = selection[3]
	y2 = selection[4]
	
	
	
	
	inpoints = []
	bright_value = 0
	dark_value = 255
	gimp.progress_init("Finding selected pixels...")
	for y in range(y1, y2):
		gimp.progress_update(1.0*(y-y1)/(y2-y1))
		for x in range(x1, x2):
			if (image.selection.get_pixel(x,y)[0] > 0):
				inpoints.append((x,y))
				pix = get_pixel((x,y),layer.width,p_size,src_pixels)
				bright_value = max(bright_value,pix[0])
				dark_value = min(dark_value,pix[0])
	pdb.gimp_message(str(bright_value) + "," + str(dark_value))
	difference = bright_value - dark_value
	new_difference = new_bright_value - new_dark_value
	gimp.progress_init("Calculate and render selected pixels...")
	for i in range(0,len(inpoints)):
		gimp.progress_update(1.0*(i)/(len(inpoints)))
		pix = get_pixel(inpoints[i],layer.width,p_size,src_pixels)
		if difference == 0:
			percentage = 0
		else:
			percentage = (pix[0] - dark_value) * 1.0 / difference
		c_value = int((percentage * new_difference) + new_dark_value)
		set_pixel(inpoints[i],layer.width,p_size,src_pixels,[c_value, c_value, c_value, pix[3]])
		
	
	srcRgn[0:layer.width, 0:layer.height] = src_pixels.tostring()
	layer.flush()
	layer.update(0,0,layer.width,layer.height)
	
	pdb.gimp_displays_flush()
	pdb.gimp_message("Done!")	
	
	pdb.gimp_image_undo_group_end(image)
	
register(
    "python_fu_height_stretch_selection",
    "Stretches heightmap of selection proportionally to new bright/dark values",
    "Stretches heightmap of selection proportionally to new bright/dark values",
    "Tin Tran",
    "GPL",
    "May 2016",
    "<Image>/Python-Fu/Height Stretch Selection...",
    "*",
    [
    #(PF_IMAGE, "image", "Input image", None), #current image
    #(PF_DRAWABLE, "layer2", "Intersect with layer", None), #current layer
    #(PF_STRING, "ok","Just OK to Continue", "OK"),
	#(PF_OPTION,"leftright",   "Direction:", 0, ["Left To Right","Right To Left"]),
	#(PF_STRING, "ok","Just OK to Continue", "OK"),
	#(PF_DRAWABLE, "layer2", "Intersect with layer", None), #another layer to be selected to intersect with
	(PF_SLIDER, "new_bright_value", "New Bright Value:", 255, (0, 255, 1)),
	(PF_SLIDER, "new_dark_value", "New Dark Value:", 0, (0, 255, 1)),
	],
    [],
    height_stretch_selection)

main()